/***************************************************************************
                          kfilereplace.h  -  description
                             -------------------
    begin                : sam oct 16 15:28:00 CEST 1999
    copyright            : (C) 1999 by Franois Dupoux
    email                : fdupoux@lemel.fr
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KFILEREPLACE_H
#define KFILEREPLACE_H
 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


// include files for KDE 
#include <kapp.h>
#include <ktmainwindow.h>
#include <kaccel.h>
#include <ktablistbox.h>

// application includes
#include "apistruct.h"

// general unix includes
//#include <pthread.h>

// forward declaration of the KFileReplace classes
class KFileReplaceDoc;
class KFileReplaceView;
class ResultView;

// Global Thread data
extern bool g_bThreadRunning;
extern bool g_bThreadMustStop;
extern int g_nFilesRep;
extern RepDirArg g_argu;
extern char g_szErrMsg[];
//extern pthread_t g_threadReplace;

/**
  * The base class for KFileReplace application windows. It sets up the main
  * window and reads the config file as well as providing a menubar, toolbar
  * and statusbar. An instance of KFileReplaceView creates your center view, which is connected
  * to the window's Doc object.
  * KFileReplaceApp reimplements the methods that KTMainWindow provides for main window handling and supports
  * full session management as well as keyboard accelerator configuration by using KAccel.
	* @see KTMainWindow
	* @see KApplication
	* @see KConfig
	* @see KAccel
	*
	* @author Source Framework Automatically Generated by KDevelop, (c) The KDevelop Team.
	* @version KDevelop version 0.4 code generation
  */
class KFileReplaceApp : public KTMainWindow
{
  Q_OBJECT

  friend class KFileReplaceView;

public:
  /** construtor of KFileReplaceApp, calls all init functions to create the application.
    * @see initMenuBar initToolBar
    */
  KFileReplaceApp();

  ~KFileReplaceApp();
  /** enables menuentries/toolbar items
  	*/
	/** gray or ungray commands
    */
	void updateCommands();

  void enableCommand(int id_,bool bState);
  /** disables menuentries/toolbar items
  	*/
	KFileReplaceDoc* getDocument() const; 	
protected:


	int checkBeforeOperation(int nTypeOfOperation);

 	/** save general Options like all bar positions and status as well as the geometry and the recent file list to the configuration
		* file
		*/ 	
	void saveOptions();
	/** read general Options again and initialize all variables like the recent file list
	  */
	void readOptions();
	/** initKeyAccel creates the keyboard accelerator items for the available slots and changes the menu accelerators.
		* @see KAccel
		*/
	void initKeyAccel();
  /** initMenuBar creates the menubar and inserts the menupopups as well as creating the helpMenu.
  	* @see KApplication#getHelpMenu
  	*/
  void initMenuBar();
  /** this creates the toolbars.
    */
  void initToolBar();
  /** sets up the statusbar for the main window by initialzing a statuslabel.
  	*/
  void initStatusBar();
	/** initializes the document object of the main window that is connected to the view in initView().
		* @see initView();
		*/
	void initDocument();
  /** creates the centerwidget of the KTMainWindow instance and sets it as the view
  	*/
  void initView();
	/** queryClose is called by KTMainWindow on each closeEvent of a window. Against the
		* default implementation (only returns true), this calles saveModified() on the document object to ask if the document shall
		* be saved if Modified; on cancel the closeEvent is rejected.
		* @see KTMainWindow#queryClose
		* @see KTMainWindow#closeEvent
		*/
	virtual bool queryClose();
	/** queryExit is called by KTMainWindow when the last window of the application is going to be closed during the closeEvent().
		* Against the default implementation that just returns true, this calls saveOptions() to save the settings of the last window's	
		* properties.
		* @see KTMainWindow#queryExit
		* @see KTMainWindow#closeEvent
		*/
	virtual bool queryExit();
  /** saves the window properties for each open window during session end to the session config file, including saving the currently
		* opened file by a temporary filename provided by KApplication.
   	* @see KTMainWindow#saveProperties
   	*/
  virtual void saveProperties(KConfig* );
  /** reads the session config file and restores the application's state including the last opened files and documents by reading the
		* temporary files saved by saveProperties()
   	* @see KTMainWindow#readProperties
  	*/
  virtual void readProperties(KConfig* );

 public slots:
  /** switch argument for slot selection by menu or toolbar ID */
  void commandCallback(int id_);
  /** switch argument for Statusbar help entries on slot selection. Add your ID's help here for toolbars and menubar entries. */
  void statusCallback(int id_);
  /** open a new application window by creating a new instance of KFileReplaceApp */
  void slotFileNewWindow();
  /** clears the document in the actual view to reuse it as the new document */
  void slotFileNew();

  void slotFileQuit();
  /** toggles the toolbar
  	*/
  void slotViewToolBar();
  /** toggles the statusbar
  	*/
  void slotViewStatusBar();
  /** changes the statusbar contents for the standard label permanently, used to indicate current actions.
  	* @param text the text that is displayed in the statusbar
  	*/
  void slotStatusMsg(const char *text);
  /** changes the status message of the whole statusbar for two seconds, then restores the last status. This is used to display
		* statusbar messages that give information about actions for toolbar icons and menuentries.
  	* @param text the text that is displayed in the statusbar
   	*/
  void slotStatusHelpMsg(const char *text);

	void slotOpenRecentStringFile(int i);
	void slotFileSearch();
	void slotFileReplace();
	void slotFileStop();
	void slotFileSave();
	void slotStringsAdd();
	void slotStringsDel();
	void slotStringsEmpty();
	void slotStringsEdit();
	void slotStringsSave();
	void slotStringsLoad();
	void slotStringsInvertCur();
	void slotStringsInvertAll();
	void slotOptionsRecursive();
	void slotOptionsBakup();
	void slotOptionsCaseSensitive();
	void slotResultInfo();
	void slotResultOpen();
	void slotResultDirOpen();
	void slotResultDelete();
	void timerEvent(QTimerEvent *);
	void closeEvent ( QCloseEvent *);
	void slotHelpWebSite();
	void slotHelpAbout();


private:
	/** the configuration object of the application */
	KConfig* config;
	/** the key accelerator container */
	KAccel* key_accel;
  /** file_menu contains all items of the menubar entry "File" */
  QPopupMenu *file_menu;
  /** edit_menu contains all items of the menubar entry "Strings" */
  QPopupMenu *strings_menu;
  /** options_menu contains all items of the menubar entry "Options" */
  QPopupMenu *options_menu;
  /** results_menu contains all items of the menubar entry "Results" */
  QPopupMenu *results_menu;
  /** view_menu contains all items of the menubar entry "View" */
  QPopupMenu *view_menu;
  /** help_menu contains all items of the menubar entry "Help" */
  QPopupMenu *help_menu;
  /** view is the main widget which represents your working area. The View
    * class should handle all events of the view widget.  It is kept empty so
    * you can create your view according to your application's needs by
    * changing the view class. */
  KFileReplaceView *view;
  /** doc represents your actual document and is created only once. It keeps
    * information such as filename and does the serialization of your files.
    */
  KFileReplaceDoc *doc;
  QPopupMenu *recentStringPopup;
  QStrList recentStringFileList; // a list of recently accessed files
	void AddRecentStringFileToList(QString strFilename);	
	void LoadStringFile(QString strFilename);
	

};


QString formatSize(uint nSize);

#endif // KFILEREPLACE_H

































